import "dart:async";

import "package:flutter/foundation.dart";
import "package:flutter_chat_interface/flutter_chat_interface.dart";

/// A class providing local chat overview service implementation.
class LocalChatOverviewService
    with ChangeNotifier
    implements ChatOverviewService {
  /// The list of currently selected users.
  final List<ChatUserModel> _currentlySelectedUsers = [];

  /// The list of personal chat models.
  final List<ChatModel> _chats = [];

  /// Retrieves the list of personal chat models.
  List<ChatModel> get chats => _chats;

  /// The stream controller for chats.
  final StreamController<List<ChatModel>> _chatsController =
      StreamController<List<ChatModel>>.broadcast();

  Future<void> updateChat(ChatModel chat) {
    var index = _chats.indexWhere((element) => element.id == chat.id);
    _chats[index] = chat;
    _chatsController.addStream(Stream.value(_chats));
    notifyListeners();
    return Future.value();
  }

  @override
  Future<void> deleteChat(ChatModel chat) {
    _chats.removeWhere((element) => element.id == chat.id);
    _chatsController.add(_chats);
    notifyListeners();
    return Future.value();
  }

  @override
  Future<ChatModel> getChatById(String id) {
    var chat = _chats.firstWhere((element) => element.id == id);
    return Future.value(chat);
  }

  @override
  Future<PersonalChatModel> getChatByUser(ChatUserModel user) async {
    PersonalChatModel? chat;
    try {
      chat = _chats
          .whereType<PersonalChatModel>()
          .firstWhere((element) => element.user.id == user.id);
      // ignore: avoid_catching_errors
    } on StateError {
      chat = PersonalChatModel(
        user: user,
        messages: [],
        id: "",
      );
      chat.id = chat.hashCode.toString();
      _chats.add(chat);
    }

    _chatsController.add([..._chats]);
    notifyListeners();
    return chat;
  }

  @override
  Stream<List<ChatModel>> getChatsStream() => _chatsController.stream;

  @override
  Stream<int> getUnreadChatsCountStream() => Stream.value(0);

  @override
  Future<void> readChat(ChatModel chat) async => Future.value();

  @override
  Future<ChatModel> storeChatIfNot(ChatModel chat, Uint8List? image) {
    var chatExists = _chats.any((element) => element.id == chat.id);

    if (!chatExists) {
      chat.id = chat.hashCode.toString();
      _chats.add(chat);
      _chatsController.add([..._chats]);
      currentlySelectedUsers.clear();
      notifyListeners();
    }

    return Future.value(chat);
  }

  @override
  List<ChatUserModel> get currentlySelectedUsers => _currentlySelectedUsers;

  @override
  void addCurrentlySelectedUser(ChatUserModel user) {
    _currentlySelectedUsers.add(user);
    notifyListeners();
  }

  @override
  void removeCurrentlySelectedUser(ChatUserModel user) {
    _currentlySelectedUsers.remove(user);
    notifyListeners();
  }

  @override
  Future<String> uploadGroupChatImage(Uint8List image, String chatId) =>
      Future.value("https://picsum.photos/200/300");

  @override
  void clearCurrentlySelectedUsers() {
    _currentlySelectedUsers.clear();
    notifyListeners();
  }
}
